@@ -2,9 +2,9 @@
 Intermediate representation (IR) for many-body propagators
 ==========================================================
 """
-__copyright__ = "2020-2021 Markus Wallerberger and others"
+__copyright__ = "2020-2022 Markus Wallerberger, Hiroshi Shinaoka, and others"
 __license__ = "MIT"
-__version__ = "0.7.1"
+__version__ = "0.90.0"
 
 min_xprec_version = "1.0"
 
@@ -19,5 +19,6 @@ except ImportError:
 
 from .kernel import RegularizedBoseKernel, LogisticKernel
 from .sve import compute as compute_sve
-from .basis import IRBasis, FiniteTempBasis, finite_temp_bases
+from .basis import DimensionlessBasis, FiniteTempBasis, finite_temp_bases
+from .basis_set import FiniteTempBasisSet
 from .sampling import TauSampling, MatsubaraSampling
