@@ -1,19 +1,13 @@
-# Copyright (C) 2020-2021 Markus Wallerberger and others
+# Copyright (C) 2020-2022 Markus Wallerberger, Hiroshi Shinaoka, and others
 # SPDX-License-Identifier: MIT
 import numpy as np
+from warnings import warn
 import numpy.polynomial.legendre as np_legendre
 import scipy.special as sp_special
 
+from . import _util
 from . import _roots
-from . import gauss
-
-try:
-    from xprec import ddouble as _ddouble
-except ImportError:
-    _ddouble = None
-    _xwork_dtype = float
-else:
-    _xwork_dtype = _ddouble
+from . import _gauss
 
 
 class PiecewiseLegendrePoly:
@@ -23,13 +17,16 @@ class PiecewiseLegendrePoly:
     intervals ``S[i] = [a[i], a[i+1]]``, where on each interval the function
     is expanded in scaled Legendre polynomials.
     """
-    def __init__(self, data, knots, dx=None):
+    def __init__(self, data, knots, dx=None, symm=None):
         """Piecewise Legendre polynomial"""
         if np.isnan(data).any():
             raise ValueError("PiecewiseLegendrePoly: data contains NaN!")
         if isinstance(knots, self.__class__):
+            if dx is not None or symm is None:
+                raise RuntimeError("wrong arguments")
             self.__dict__.update(knots.__dict__)
             self.data = data
+            self.symm = symm
             return
 
         data = np.array(data)
@@ -39,6 +36,13 @@ class PiecewiseLegendrePoly:
             raise ValueError("Invalid knots array")
         if not (knots[1:] >= knots[:-1]).all():
             raise ValueError("Knots must be monotonically increasing")
+        if symm is None:
+            # TODO: infer symmetry from data
+            symm = np.zeros(data.shape[2:])
+        else:
+            symm = np.array(symm)
+            if symm.shape != data.shape[2:]:
+                raise ValueError("shape mismatch")
         if dx is None:
             dx = knots[1:] - knots[:-1]
         else:
@@ -54,17 +58,19 @@ class PiecewiseLegendrePoly:
         self.knots = knots
         self.dx = dx
         self.data = data
+        self.symm = symm
         self._xm = .5 * (knots[1:] + knots[:-1])
         self._inv_xs = 2/dx
         self._norm = np.sqrt(self._inv_xs)
 
     def __getitem__(self, l):
         """Return part of a set of piecewise polynomials"""
+        new_symm = self.symm[l]
         if isinstance(l, tuple):
             new_data = self.data[(slice(None), slice(None), *l)]
         else:
             new_data = self.data[:,:,l]
-        return self.__class__(new_data, self)
+        return self.__class__(new_data, self, symm=new_symm)
 
     def __call__(self, x):
         """Evaluate polynomial at position x"""
@@ -96,7 +102,7 @@ class PiecewiseLegendrePoly:
         res *= self._norm[i]
         return res
 
-    def overlap(self, f, axis=None, _deg=None):
+    def overlap(self, f, *, rtol=2.3e-16, return_error=False):
         r"""Evaluate overlap integral of this polynomial with function ``f``.
 
         Given the function ``f``, evaluate the integral::
@@ -109,33 +115,18 @@ class PiecewiseLegendrePoly:
         Arguments:
             f (callable):
                 function that is called with a point ``x`` and returns ``f(x)``
-                at that position.  If the ``axis`` argument is given, ``f``
-                must be vectorized.
-            axis (int or None):
-                If `None` (the default), `f` is called repeatedly for all
-                quadrature points.  If `axis` is not None, `f`, when called
-                for a vector `x` returns `f(x[i])` at the `i`-th position along
-                the given axis.
+                at that position.
 
         Return:
             array-like object with shape (poly_dims, f_dims)
             poly_dims are the shape of the polynomial and f_dims are those
             of the function f(x).
         """
-        if _deg is None:
-            _deg = 2*self.polyorder
-
-        # Get Gauss rule
-        rule = gauss.legendre(_deg, self.data.dtype).piecewise(self.knots)
-        x = rule.x
-
-        # Multiply weights by polynomial at value
-        pw = self(x) * rule.w
-        fx = _apply_along_axis(f, x, axis)
-
-        # Perform the summation and reshape the result
-        int_flat = pw.reshape(self.size, x.size) @ fx.reshape(x.size, -1)
-        return np.asarray(int_flat).reshape(self.shape + fx.shape[1:])
+        int_result, int_error = _compute_overlap(self, f, rtol=rtol)
+        if return_error:
+            return int_result, int_error
+        else:
+            return int_result
 
     def deriv(self, n=1):
         """Get polynomial for the n'th derivative"""
@@ -144,7 +135,7 @@ class PiecewiseLegendrePoly:
         _scale_shape = (1, -1) + (1,) * (self.data.ndim - 2)
         scale = self._inv_xs ** n
         ddata *= scale.reshape(_scale_shape)
-        return self.__class__(ddata, self.knots, self.dx)
+        return self.__class__(ddata, self, symm=(-1)**n * self.symm)
 
     def hat(self, freq, n_asymp=None):
         """Get Fourier transformed object"""
@@ -159,8 +150,30 @@ class PiecewiseLegendrePoly:
         if self.data.ndim > 2:
             raise ValueError("select single polynomial before calling roots()")
 
-        grid = _refine_grid(self.knots, alpha)
-        return _roots.find_all(self, grid)
+        grid = self.knots
+        xmid = (self.xmax + self.xmin) / 2
+        if self.symm:
+            if grid[self.nsegments // 2] == xmid:
+                grid = grid[self.nsegments//2:]
+            else:
+                grid = np.hstack((xmid, grid[grid > xmid]))
+
+        grid = _refine_grid(grid, alpha)
+        roots = _roots.find_all(self, grid)
+
+        if self.symm == 1:
+            revroots = (self.xmax + self.xmin) - roots[::-1]
+            roots = np.hstack((revroots, roots))
+        elif self.symm == -1:
+            # There must be a zero at exactly the midpoint, but we may either
+            # slightly miss it or have a spurious zero
+            if roots.size:
+                if roots[0] == xmid or self(xmid) * self.deriv()(xmid) < 0:
+                    roots = roots[1:]
+            revroots = (self.xmax + self.xmin) - roots[::-1]
+            roots = np.hstack((revroots, xmid, roots))
+
+        return roots
 
     @property
     def shape(self): return self.data.shape[2:]
@@ -201,7 +214,7 @@ class PiecewiseLegendreFT:
     _DEFAULT_GRID = np.hstack([np.arange(2**6),
                         (2**np.linspace(6, 25, 16*(25-6)+1)).astype(int)])
 
-    def __init__(self, poly, freq='even', n_asymp=None):
+    def __init__(self, poly, freq='even', n_asymp=None, power_model=None):
         if poly.xmin != -1 or poly.xmax != 1:
             raise NotImplementedError("Only interval [-1, 1] supported")
         self.poly = poly
@@ -212,7 +225,10 @@ class PiecewiseLegendreFT:
             self._model = None
         else:
             self.n_asymp = n_asymp
-            self._model = _power_model(freq, poly)
+            if power_model is None:
+                self._model = _power_model(freq, poly)
+            else:
+                self._model = power_model
 
     @property
     def shape(self): return self.poly.shape
@@ -224,15 +240,23 @@ class PiecewiseLegendreFT:
     def ndim(self): return self.poly.ndim
 
     def __getitem__(self, l):
-        return self.__class__(self.poly[l], self.freq)
+        model = self._model if self._model is None else self._model[l]
+        return self.__class__(self.poly[l], self.freq, self.n_asymp, model)
 
     def __call__(self, n):
+        """Obtain Fourier transform of polynomial for given frequencies"""
+        # Evaluate only on unique frequencies
+        n_unique, n_where = np.unique(n.ravel(), return_inverse=True)
+        result_flat = self._call_impl(n_unique)[..., n_where]
+        return result_flat.reshape(self.poly.shape + n.shape)
+
+    def _call_impl(self, n):
         """Obtain Fourier transform of polynomial for given frequencies"""
         n = check_reduced_matsubara(n, self.zeta)
         n_flat = n.ravel()
         result_flat = _compute_unl_inner(self.poly, n_flat)
 
-        # We use use the asymptotics at frequencies larger than conv_radius
+        # We use the asymptotics at frequencies larger than conv_radius
         # since it has lower relative error.
         cond_outer = np.abs(n_flat) >= self.n_asymp
         if cond_outer.any():
@@ -249,13 +273,13 @@ class PiecewiseLegendreFT:
             grid = self._DEFAULT_GRID
 
         f = self._func_for_part(part)
-        x0 = _roots.discrete_extrema(f, PiecewiseLegendreFT._DEFAULT_GRID)
+        x0 = _roots.discrete_extrema(f, grid)
         x0 = 2 * x0 + self.zeta
         return _symmetrize_matsubara(x0)
 
     def _func_for_part(self, part=None):
         if part is None:
-            parity = self.poly(self.poly.xmax) / self.poly(self.poly.xmin)
+            parity = self.poly.symm
             if np.allclose(parity, 1):
                 part = 'real' if self.zeta == 0 else 'imag'
             elif np.allclose(parity, -1):
@@ -378,7 +402,7 @@ def _compute_unl_inner(poly, wn):
     # Perform the following, but faster:
     #   resulth = einsum('pin,pil->nl', t_pin, data_sc)
     npi = poly.polyorder * poly.nsegments
-    result_flat = t_pin.reshape(npi,-1).T.dot(data_sc.reshape(npi,-1)).T
+    result_flat = (t_pin.reshape(npi,-1).T @ data_sc.reshape(npi,-1)).T
     return result_flat.reshape(*poly.data.shape[2:], wn.size)
 
 
@@ -392,11 +416,15 @@ class _PowerModel:
     """
     def __init__(self, moments):
         """Initialize model"""
+        if moments.ndim == 1:
+            moments = moments[:, None]
         self.moments = np.asarray(moments)
         self.nmom, self.nl = self.moments.shape
 
-    def _giw_ravel(self, wn):
+    @_util.ravel_argument()
+    def giw(self, wn):
         """Return model Green's function for vector of frequencies"""
+        wn = check_reduced_matsubara(wn)
         result_dtype = np.result_type(1j, wn, self.moments)
         result = np.zeros((wn.size, self.nl), result_dtype)
         inv_iw = 1j * np.pi/2 * wn
@@ -406,10 +434,8 @@ class _PowerModel:
             result *= inv_iw[:, None]
         return result
 
-    def giw(self, wn):
-        """Return model Green's function for reduced frequencies"""
-        wn = check_reduced_matsubara(wn)
-        return self._giw_ravel(wn.ravel()).reshape(wn.shape + (self.nl,))
+    def __getitem__(self, l):
+        return self.__class__(self.moments[:,l])
 
 
 def _derivs(ppoly, x):
@@ -444,21 +470,6 @@ def _refine_grid(knots, alpha):
     return np.hstack((result.T.ravel(), knots[-1]))
 
 
-def _apply_along_axis(f, x, axis=None):
-    if axis is None:
-        fx = [f(xi) for xi in x]
-        fx = np.array(fx)
-        if fx.dtype is np.dtype(object):
-            raise ValueError("incompatible shapes")
-    else:
-        fx = np.asarray(f(x))
-        if fx.shape[axis] != x.size:
-            raise ValueError("inconsistent result shape")
-        if axis != 0:
-            fx = np.moveaxis(fx, axis, 0)
-    return fx
-
-
 def _symmetrize_matsubara(x0):
     if not (x0[1:] >= x0[:-1]).all():
         raise ValueError("set of Matsubara points not ordered")
@@ -469,3 +480,57 @@ def _symmetrize_matsubara(x0):
     else:
         x0 = np.hstack([-x0[::-1], x0])
     return x0
+
+
+def _compute_overlap(poly, f, rtol=2.3e-16, radix=2, max_refine_levels=40,
+                     max_refine_points=2000):
+    base_rule = _gauss.kronrod_31_15()
+    xstart = poly.knots[:-1]
+    xstop = poly.knots[1:]
+
+    f_shape = None
+    res_value = 0
+    res_error = 0
+    res_magn = 0
+    for _ in range(max_refine_levels):
+        #print(f"Level {_}: {xstart.size} segments")
+        if xstart.size > max_refine_points:
+            warn("Refinement is too broad, aborting (increase rtol)")
+            break
+
+        rule = base_rule.reseat(xstart[:, None], xstop[:, None])
+
+        fx = np.array(list(map(f, rule.x.ravel())))
+        if f_shape is None:
+            f_shape = fx.shape[1:]
+        elif fx.shape[1:] != f_shape:
+            raise ValueError("inconsistent shapes")
+        fx = fx.reshape(rule.x.shape + (-1,))
+
+        valx = poly(rule.x).reshape(-1, *rule.x.shape, 1) * fx
+        int21 = (valx[:, :, :, :] * rule.w[:, :, None]).sum(2)
+        int10 = (valx[:, :, rule.vsel, :] * rule.v[:, :, None]).sum(2)
+        intdiff = np.abs(int21 - int10)
+        intmagn = np.abs(int10)
+
+        magn = res_magn + intmagn.sum(1).max(1)
+        relerror = intdiff.max(2) / magn[:, None]
+
+        xconverged = (relerror <= rtol).all(0)
+        res_value += int10[:, xconverged].sum(1)
+        res_error += intdiff[:, xconverged].sum(1)
+        res_magn += intmagn[:, xconverged].sum(1).max(1)
+        if xconverged.all():
+            break
+
+        xrefine = ~xconverged
+        xstart = xstart[xrefine]
+        xstop = xstop[xrefine]
+        xedge = np.linspace(xstart, xstop, radix + 1, axis=-1)
+        xstart = xedge[:, :-1].ravel()
+        xstop = xedge[:, 1:].ravel()
+    else:
+        warn("Integration did not converge after refinement")
+
+    res_shape = poly.shape + f_shape
+    return res_value.reshape(res_shape), res_error.reshape(res_shape)
